/// <reference path="./css.d.ts"/>
"use strict";
/**
 * @module Maintains a single stylesheet and keeps it in sync with requested styles
 */
var FreeStyle = require("free-style");
/**
 * Only calls cb all sync operations settle
 */
var afterAllSync = (new (function () {
    function class_1() {
        var _this = this;
        this.afterAllSync = function (cb) {
            if (_this.pending)
                clearTimeout(_this.pending);
            _this.pending = setTimeout(cb);
        };
    }
    return class_1;
}())).afterAllSync;
/**
 * We have a single stylesheet that we update as components register themselves
 */
var freeStyle = FreeStyle.create();
var lastChangeId = freeStyle.changeId;
var singletonTag = typeof window === 'undefined' ? { innerHTML: '' } : document.createElement('style');
if (typeof document !== 'undefined')
    document.head.appendChild(singletonTag);
/** Checks if the style tag needs updating and if so queues up the change */
var styleUpdated = function () {
    if (freeStyle.changeId === lastChangeId)
        return;
    lastChangeId = freeStyle.changeId;
    afterAllSync(function () {
        singletonTag.innerHTML = freeStyle.getStyles();
    });
};
/**
 * Helps with testing. Reinitializes FreeStyle
 */
function reinit() {
    freeStyle = FreeStyle.create();
    lastChangeId = freeStyle.changeId;
    singletonTag.innerHTML = '';
}
exports.reinit = reinit;
/**
 * Allows use to use the stylesheet in a node.js environment
 */
exports.css = function () { return freeStyle.getStyles(); };
/**
 * Takes CSSProperties and return a generated className you can use on your component
 */
function style() {
    var objects = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        objects[_i - 0] = arguments[_i];
    }
    var object = extend.apply(void 0, objects);
    var className = freeStyle.registerStyle(object);
    styleUpdated();
    return className;
}
exports.style = style;
/**
 * Takes Keyframes and returns a generated animation name
 */
function keyframes(frames) {
    var animationName = freeStyle.registerKeyframes(frames);
    styleUpdated();
    return animationName;
}
exports.keyframes = keyframes;
/**
 * Merges various styles into a single style object.
 * Note: if two objects have the same property the last one wins
 */
function extend() {
    var objects = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        objects[_i - 0] = arguments[_i];
    }
    /** The final result we will return */
    var result = {};
    for (var _a = 0, objects_1 = objects; _a < objects_1.length; _a++) {
        var object = objects_1[_a];
        for (var key in object) {
            if (
            // Some psuedo state or media query
            (key.indexOf('&') !== -1 || key.indexOf('@media') === 0)
                && result[key]) {
                // Then extend in the final result
                result[key] = extend(result[key], object);
            }
            else {
                result[key] = object[key];
            }
        }
    }
    return result;
}
exports.extend = extend;
/**
 * Utility to join classes conditionally
 */
function classes() {
    var classes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        classes[_i - 0] = arguments[_i];
    }
    return classes.filter(function (c) { return !!c; }).join(' ');
}
exports.classes = classes;
